/* global $, document, console, window, setTimeout */
/*globals $, console, jQuery, fwColorPicker */
/*jslint bitwise: true */
/*jslint sub:true*/
/**!
 * @preserve Color animation 1.6.0
 * http://www.bitstorm.org/jquery/color-animation/
 * Copyright 2011, 2013 Edwin Martin
 * Released under the MIT and GPL licenses.
 */

(function ($) {
  "use strict";

  // Some named colors to work with, added by Bradley Ayers
  // From Interface by Stefan Petre
  // http://interface.eyecon.ro/
  var colors, properties;
  colors = {
    'aqua': [0, 255, 255, 1],
    'azure': [240, 255, 255, 1],
    'beige': [245, 245, 220, 1],
    'black': [0, 0, 0, 1],
    'blue': [0, 0, 255, 1],
    'brown': [165, 42, 42, 1],
    'cyan': [0, 255, 255, 1],
    'darkblue': [0, 0, 139, 1],
    'darkcyan': [0, 139, 139, 1],
    'darkgrey': [169, 169, 169, 1],
    'darkgreen': [0, 100, 0, 1],
    'darkkhaki': [189, 183, 107, 1],
    'darkmagenta': [139, 0, 139, 1],
    'darkolivegreen': [85, 107, 47, 1],
    'darkorange': [255, 140, 0, 1],
    'darkorchid': [153, 50, 204, 1],
    'darkred': [139, 0, 0, 1],
    'darksalmon': [233, 150, 122, 1],
    'darkviolet': [148, 0, 211, 1],
    'fuchsia': [255, 0, 255, 1],
    'gold': [255, 215, 0, 1],
    'green': [0, 128, 0, 1],
    'indigo': [75, 0, 130, 1],
    'khaki': [240, 230, 140, 1],
    'lightblue': [173, 216, 230, 1],
    'lightcyan': [224, 255, 255, 1],
    'lightgreen': [144, 238, 144, 1],
    'lightgrey': [211, 211, 211, 1],
    'lightpink': [255, 182, 193, 1],
    'lightyellow': [255, 255, 224, 1],
    'lime': [0, 255, 0, 1],
    'magenta': [255, 0, 255, 1],
    'maroon': [128, 0, 0, 1],
    'navy': [0, 0, 128, 1],
    'olive': [128, 128, 0, 1],
    'orange': [255, 165, 0, 1],
    'pink': [255, 192, 203, 1],
    'purple': [128, 0, 128, 1],
    'violet': [128, 0, 128, 1],
    'red': [255, 0, 0, 1],
    'silver': [192, 192, 192, 1],
    'white': [255, 255, 255, 1],
    'yellow': [255, 255, 0, 1],
    'transparent': [255, 255, 255, 0]
  };
  function isRGBACapable() {
    var $script = $('script:first'),
      color = $script.css('color'),
      result = false;
    if (/^rgba/.test(color)) {
      result = true;
    } else {
      try {
        result = (color !== $script.css('color', 'rgba(0, 0, 0, 0.5)').css('color'));
        $script.css('color', color);
      } catch (e) { }
    }

    return result;
  }

  // Calculate an in-between color. Returns "#aabbcc"-like string.
  function calculateColor(begin, end, pos) {
    var color = 'rgb' + ($.support['rgba'] ? 'a' : '') + '('
      + parseInt((begin[0] + pos * (end[0] - begin[0])), 10) + ','
      + parseInt((begin[1] + pos * (end[1] - begin[1])), 10) + ','
      + parseInt((begin[2] + pos * (end[2] - begin[2])), 10);
    if ($.support['rgba']) {
      color += ',' + (begin && end ? parseFloat(begin[3] + pos * (end[3] - begin[3])) : 1);
    }
    color += ')';
    return color;
  }

  // Parse an CSS-syntax color. Outputs an array [r, g, b]
  function parseColor(color) {
    var match, quadruplet;

    // Match #aabbcc
    if (match = /#([0-9a-fA-F]{2})([0-9a-fA-F]{2})([0-9a-fA-F]{2})/.exec(color)) {
      quadruplet = [parseInt(match[1], 16), parseInt(match[2], 16), parseInt(match[3], 16), 1];

	// Match #abc
    } else if (match = /#([0-9a-fA-F])([0-9a-fA-F])([0-9a-fA-F])/.exec(color)) {
      quadruplet = [parseInt(match[1], 16) * 17, parseInt(match[2], 16) * 17, parseInt(match[3], 16) * 17, 1];

    // Match rgb(n, n, n)
    } else if (match = /rgb\(\s*([0-9]{1,3})\s*,\s*([0-9]{1,3})\s*,\s*([0-9]{1,3})\s*\)/.exec(color)) {
      quadruplet = [parseInt(match[1]), parseInt(match[2]), parseInt(match[3]), 1];

    } else if (match = /rgba\(\s*([0-9]{1,3})\s*,\s*([0-9]{1,3})\s*,\s*([0-9]{1,3})\s*,\s*([0-9\.]*)\s*\)/.exec(color)) {
      quadruplet = [parseInt(match[1], 10), parseInt(match[2], 10), parseInt(match[3], 10), parseFloat(match[4])];

      // No browser returns rgb(n%, n%, n%), so little reason to support this format.
    } else {
      quadruplet = colors[color];
    }
    return quadruplet;
  }
  
  $.extend(true, $, {
    support: {
      'rgba': isRGBACapable()
    }
  });

  properties = ['color', 'backgroundColor', 'borderBottomColor', 'borderLeftColor', 'borderRightColor', 'borderTopColor', 'outlineColor'];
  
  $.each(properties, function (i, property) {
    $.Tween.propHooks[property] = {
      get: function (tween) {
        return $(tween.elem).css(property);
      },
      set: function (tween) {
        var style, p_begin, p_end;
        style = tween.elem.style;
        p_begin = parseColor($(tween.elem).css(property));
        p_end = parseColor(tween.end);
        tween.run = function (progress) {
          style[property] = calculateColor(p_begin, p_end, progress);
        };
      }
    };
  });

  // borderColor doesn't fit in standard fx.step above.
  $.Tween.propHooks.borderColor = {
    set: function (tween) {
      var style, p_begin, p_end, borders;
      style = tween.elem.style;
      p_begin = [];
      borders = properties.slice(2, 6); // All four border properties
      $.each(borders, function (i, property) {
        p_begin[property] = parseColor($(tween.elem).css(property));
      });
      p_end = parseColor(tween.end);
      tween.run = function (progress) {
        $.each(borders, function (i, property) {
          style[property] = calculateColor(p_begin[property], p_end, progress);
        });
      };
    }
  };
})(jQuery);
jQuery(document).ready(function ($) {
  "use strict";
  (function (window) {
    var fwColorPicker = (function () {
      var CP = null, colorInput, r = 0, g = 0, b = 0, a = 100, animateFW;
      
      //rgb to hsv
      function rgb2hsv(r, g, b) {
        var rabs, gabs, babs, rr, gg, bb, h, s, v, diff, diffc, c, percentRoundFn;
        rabs = r / 255;
        gabs = g / 255;
        babs = b / 255;
        v = Math.max(rabs, gabs, babs);
        diff = v - Math.min(rabs, gabs, babs);
        diffc = c => (v - c) / 6 / diff + 1 / 2;
        percentRoundFn = num => Math.round(num * 100) / 100;
        if (diff == 0) {
            h = s = 0;
        } else {
            s = diff / v;
            rr = diffc(rabs);
            gg = diffc(gabs);
            bb = diffc(babs);

            if (rabs === v) {
                h = bb - gg;
            } else if (gabs === v) {
                h = (1 / 3) + rr - bb;
            } else if (babs === v) {
                h = (2 / 3) + gg - rr;
            }
            if (h < 0) {
                h += 1;
            }else if (h > 1) {
                h -= 1;
            }
        }
        return {
            h: Math.round(h * 360),
            s: percentRoundFn(s * 100),
            v: percentRoundFn(v * 100)
        };
      }
      // hex 2 rgb
      function hex2rgb(hex) {
        // если введено три символа без #
        if (hex.length === 3) {
          hex += hex;
        }
        // если введено три символа c #
        if (hex.length === 4) {
          hex += hex.substr(1);
        }
        var result = /^#?([a-f\d]{2})([a-f\d]{2})([a-f\d]{2})$/i.exec(hex);
        return result ? {
          r: parseInt(result[1], 16),
          g: parseInt(result[2], 16),
          b: parseInt(result[3], 16)
        } : null;
      }
      // byte2hex
      function byte2Hex(n) {
        var nybHexString = "0123456789ABCDEF";
        return String(nybHexString.substr((n >> 4) & 0x0F, 1)) + nybHexString.substr(n & 0x0F, 1);
      }
      // rgb to hex
      function rgb2hex(r, g, b) {
        return '#' + byte2Hex(r) + byte2Hex(g) + byte2Hex(b);
      }
      // generate gradient
      function gradient(color) {
        return "background: -moz-linear-gradient(left, " + color + "); background: -o-linear-linear-gradient(left, " + color + "); background: linear-gradient(to right, " + color + "); background: -webkit-linear-gradient(left, " + color + ");";
      }

      function init() {
        var code = "";
        // transparent container
        $(".fw_color").wrap("<div class='fwcp_transparent'></div>");
        // colorize
        $(".fw_color").each(function () {
          $(this).css("background", $(this).val());
        });

        // code
        code += '<div class="fwcp_wrapper">';
        code += '  <div class="fwcp_header">';
        code += '    <div class="fwcp_row">';
        code += '      <div class="fwcp_col fwcp_heading">Select Color:</div>';
        code += '      <div class="fwcp_col fwcp_close_col"><div class="fwcp_close_icon"></div></div>';
        code += '    </div>';
        code += '  </div>';
        code += '  <div class="fwcp_content">';
        code += '    <div class="fwcp_preview"><input type="text" value="0" id="fwcp_hex"></div>';
        //red
        code += '    <div class="fwcp_row">';
        code += '      <label class="fwcp_col left_side" for="fwcp_red_value">R</label>';
        code += '      <div class="fwcp_col">';
        code += '        <div class="fwcp_slider">';
        code += '          <input tabindex="-1" class="fwcp_slider_range" type="range" step="1" min="0" max="255" value="0"  id="fwcp_red_slider">';
        code += '          <div class="fwcp_slider_handle"></div>';
        code += '        </div>';
        code += '      </div>';
        code += '      <div class="fwcp_col right_side">';
        code += '        <input type="text" id="fwcp_red_value" class="fwcp_value" value="0">';
        code += '      </div>';
        code += '    </div>';
        //green
        code += '    <div class="fwcp_row">';
        code += '      <label class="fwcp_col left_side" for="fwcp_green_value">G</label>';
        code += '      <div class="fwcp_col">';
        code += '        <div class="fwcp_slider">';
        code += '          <input tabindex="-1" class="fwcp_slider_range" type="range" step="1" min="0" max="255" value="0"  id="fwcp_green_slider">';
        code += '          <div class="fwcp_slider_handle"></div>';
        code += '        </div>';
        code += '      </div>';
        code += '      <div class="fwcp_col right_side">';
        code += '        <input type="text" id="fwcp_green_value" class="fwcp_value" value="0">';
        code += '      </div>';
        code += '    </div>';
        //blue
        code += '    <div class="fwcp_row">';
        code += '      <label class="fwcp_col left_side" for="fwcp_blue_value">B</label>';
        code += '      <div class="fwcp_col">';
        code += '        <div class="fwcp_slider">';
        code += '          <input tabindex="-1" class="fwcp_slider_range" type="range" step="1" min="0" max="255" value="0"  id="fwcp_blue_slider">';
        code += '          <div class="fwcp_slider_handle"></div>';
        code += '        </div>';
        code += '      </div>';
        code += '      <div class="fwcp_col right_side">';
        code += '        <input type="text" id="fwcp_blue_value" class="fwcp_value" value="0">';
        code += '      </div>';
        code += '    </div>';
        //alpha
        code += '    <div class="fwcp_row">';
        code += '      <label class="fwcp_col left_side" for="fwcp_alpha_value">A</label>';
        code += '      <div class="fwcp_col">';
        code += '        <div class="fwcp_slider">';
        code += '          <input tabindex="-1" class="fwcp_slider_range" type="range" step="1" min="0" max="100" value="100"  id="fwcp_alpha_slider">';
        code += '          <div class="fwcp_slider_handle"></div>';
        code += '        </div>';
        code += '      </div>';
        code += '      <div class="fwcp_col right_side">';
        code += '        <input type="text" id="fwcp_alpha_value" class="fwcp_value" value="0">';
        code += '      </div>';
        code += '    </div>';
        code += '  </div>';
        code += '  <div class="fwcp_footer">';
        code += '    <button class="fwcp_cancel fwcp_btn">Cancel</button>';
        code += '    <button class="fwcp_save fwcp_btn">Save</button>';
        code += '  </div>';
        code += '</div>';
        $("body").append(code);
        CP = $("body").find(".fwcp_wrapper");

        // add actions
        // colorize ranges
        function reangesColor() {
          var gradientRed, gradientGreen, gradientBlue, gradientAlpha;
          gradientRed = "rgb(0, " + g + ", " + b + ") 0%, rgb(" + 255 + ", " + g + ", " + b + ") 100%";
          gradientGreen = "rgb(" + r + ", 0, " + b + ") 0%, rgb(" + r + ", " + 255 + ", " + b + ") 100%";
          gradientBlue = "rgb(" + r + ", " + g + ", 0) 0%, rgb(" + r + ", " + g + ", " + 255 + ") 100%";
          gradientAlpha = "rgba(" + r + ", " + g + ", " + b + ", 0) 0%, rgba(" + r + ", " + g + ", " + b + ", 1) 100%";
          $("#fwcp_red_slider").parent().attr("style", gradient(gradientRed));
          $("#fwcp_green_slider").parent().attr("style", gradient(gradientGreen));
          $("#fwcp_blue_slider").parent().attr("style", gradient(gradientBlue));
          $("#fwcp_alpha_slider").parent().attr("style", gradient(gradientAlpha));
        }
        // change sliders events
        function changeSlider() {
          $(".fwcp_slider_range").each(function () {
            var max = $(this).attr("max"), percent = 0, hsv;
            // получаем процент
            percent = ($(this).val() / max) * 100;
            // перемещаем хендл и заем ему марджин
            $(this).next().css({
              "left": percent + "%",
              "margin-left": -0.24 * percent
            });
            // получаем значение
            r = $("#fwcp_red_slider").val();
            g = $("#fwcp_green_slider").val();
            b = $("#fwcp_blue_slider").val();
            a = $("#fwcp_alpha_slider").val();
            // вводим значение в инпут цвета
            $("#fwcp_red_value").val(r);
            $("#fwcp_green_value").val(g);
            $("#fwcp_blue_value").val(b);
            $("#fwcp_alpha_value").val(a);
            // красим ренджи
            reangesColor();
            // hex color
            $("#fwcp_hex").val(rgb2hex(r, g, b)).css("background", "rgba(" + r + ", " + g + ", " + b + ", " + (a / 100) + ")");
            // меняем цвет текста на светлый или темный
            hsv = rgb2hsv(r, g, b);
            if (hsv.v < 50) {
              $("#fwcp_hex").css("color", "#fff");
            } else {
              $("#fwcp_hex").css("color", "#000");
            }
          });
        }
        //red slider
        $("#fwcp_red_slider").on("change input", function () {
          changeSlider();
        });
        //green slider
        $("#fwcp_green_slider").on("change input", function () {
          changeSlider();
        });
        //blue slider
        $("#fwcp_blue_slider").on("change input", function () {
          changeSlider();
        });
        //alpha slider
        $("#fwcp_alpha_slider").on("change input", function () {
          a = $(this).val();
          $("#fwcp_alpha_value").val(a);
          // перемещаем хендл и заем ему марджин
          $(this).next().css({
            "left": a + "%",
            "margin-left": -0.24 * a
          });
          // hex color
          $("#fwcp_hex").css("background", "rgba(" + r + ", " + g + ", " + b + ", " + (a / 100) + ")");
          // меняем цвет текста на светлый или темный
          var hsv = rgb2hsv(r, g, b);
          if (hsv.v < 50) {
            $("#fwcp_hex").css("color", "#fff");
          } else {
            $("#fwcp_hex").css("color", "#000");
          }
        });

        //rgb values
        $("#fwcp_red_value").on("change", function () {
          $("#fwcp_red_slider").val($(this).val()).change();
        });
        $("#fwcp_green_value").on("change", function () {
          $("#fwcp_green_slider").val($(this).val()).change();
        });
        $("#fwcp_blue_value").on("change", function () {
          $("#fwcp_blue_slider").val($(this).val()).change();
        });
        $("#fwcp_alpha_value").on("change", function () {
          $("#fwcp_alpha_slider").val($(this).val()).change();
        });

        // hex change
        $("#fwcp_hex").on("change", function () {
          var rgb = hex2rgb($(this).val());
          $("#fwcp_red_slider").val(rgb.r).change();
          $("#fwcp_green_slider").val(rgb.g).change();
          $("#fwcp_blue_slider").val(rgb.b).change();
        });

        $(".fwcp_save").on("click", function () {
          // fix когда изменяем цвет, блокируем измененияцвета через стороние скрипты
          animateFW = true;
          var color = null;
          if ($("#fwcp_alpha_value").val() < 100) {
            // generate rgba
            color = "rgba(" + r + ", " + g + ", " + b + ", " + (a / 100) + ")";
          } else {
            //generate hex
            color = rgb2hex(r, g, b);
          }
          //console.log(color);
          // insert color
          colorInput.val(color).change().animate({
            backgroundColor: color
          }, 500);
          // close CP
          fwColorPicker.close();
          animateFW = false;
        });
        // клик по кенселу
        $(".fwcp_close_icon, .fwcp_cancel").on("click", function () {
          fwColorPicker.close();
        });

        // если изменили значение цвета из вне
        $(".fw_color").on("change", function () {
          if (!animateFW) {
            $(this).css("background", $(this).val());
          }
        });

      }

      function open(options) {
        var scrollTop, inputOffset, cpWidth, cpHeight, rgb;
        // заполняем переменные
        colorInput = options.input;
        scrollTop = $(document).scrollTop(); // скрол с верху
        inputOffset = colorInput.offset(); // расположение инпута
        cpWidth = CP.width(); // ширина окна
        cpHeight = CP.height(); // высота окна

        // проверям если hex
        if (colorInput.val().indexOf("#") !== -1) {
          // устанавливаем в пикер
          $("#fwcp_hex").val(colorInput.val()).change();
          $("#fwcp_alpha_slider").val(100).change();
        // проверям если rgba
        } else if (colorInput.val().indexOf("rgba") !== -1) {
          // разбиваем на массив
          rgb = colorInput.val().substring(5, colorInput.val().length - 1).replace(/ /g, '').split(',');
          // устанавливаем в пикер
          $("#fwcp_red_slider").val(rgb[0]).change();
          $("#fwcp_green_slider").val(rgb[1]).change();
          $("#fwcp_blue_slider").val(rgb[2]).change();
          $("#fwcp_alpha_slider").val(rgb[3] * 100).change();
        // проверям если rgb
        } else if (colorInput.val().indexOf("rgb") !== -1) {
          // разбиваем на массив
          rgb = colorInput.val().substring(4, colorInput.val().length - 1).replace(/ /g, '').split(',');
          // устанавливаем в пикер
          $("#fwcp_red_slider").val(rgb[0]).change();
          $("#fwcp_green_slider").val(rgb[1]).change();
          $("#fwcp_blue_slider").val(rgb[2]).change();
          $("#fwcp_alpha_slider").val(100).change();
        }
        // устанавливаем начальную позицию элемента
        CP.css({
          width: 40,
          height: 40,
          left: inputOffset.left + 22.5,
          top: inputOffset.top - scrollTop + 22.5
        });
        
        // анимируем открытие
        setTimeout(function () {
          CP.addClass("open").animate({
            width: cpWidth,
            height: cpHeight,
            top: $(window).height() / 2,
            left: $(window).width() / 2
          }, 0, "easeOutQuint");
        }, 0);
        // перемещаем окно и при изменение размеров вьювера
        $(window).resize(function () {
          CP.css({
            top: $(window).height() / 2,
            left: $(window).width() / 2
          });
        });
      }

      function close() {
        CP.addClass("close");
        setTimeout(function () {
          CP.removeClass("open");
        }, 250);
        setTimeout(function () {
          CP.removeClass("close");
        }, 400);
      }

      return {
        init: init,
        open: open,
        close: close
      };
    }());

    window.fwColorPicker = fwColorPicker;
  }(window));

});

jQuery(document).ready(function ($) {
  "use strict";
  fwColorPicker.init();
  
  $(".fw_color").on("click", function () {
    fwColorPicker.open({
      input: $(this)
    });
  });
  
});
