<?php
// Register and load the widget
function wptpa_load_widget() {
  register_widget('wptpa_widget');
}
add_action('widgets_init', 'wptpa_load_widget');
 
// Creating the widget 
class wptpa_widget extends WP_Widget {
  function __construct() {
    parent::__construct(
      // Base ID of your widget
      'wptpa_widget', 
      // Widget name will appear in UI
      __('tPlayer', 'wptpa_widget'), 
      // Widget description
      array(
        'description' => 'Widget for insert tPlayer.' 
      ) 
    );
  }

  // Creating widget front-end

  public function widget($args, $instance) {
    $id = $instance["pls_id"];
    $autoplay = $instance["autoplay"] ? 'true' : 'false';
    $playlist = $instance["playlist"] ? 'true' : 'false';
    $cover = $instance["cover"] ? 'true' : 'false';
    
    $title = apply_filters('widget_title', $instance['title']);
    // before and after widget arguments are defined by themes
    echo $args['before_widget'];
    if (!empty($title)){
      echo $args['before_title'].$title.$args['after_title'];
    }

    // This is where you run the code and display the output
    echo do_shortcode("[wptpa id='$id' autoplay='$autoplay' cover='$cover' playlist='$playlist']");
    echo $args['after_widget'];
  }

  // Widget Backend 
  public function form($instance) {
    global $wpdb;                     
    $pls_table_name = $wpdb->prefix."tpa_playlist";
    $stats_table_name = $wpdb->prefix."tpa_statistics";
    $songs_table_name = $wpdb->prefix."tpa_songs";
    if(isset($instance['title'])) {
      $title = $instance['title'];
    } else {
      $title = 'New title';
    }
    if(isset($instance['pls_id'])) {
      $pls_id = $instance['pls_id'];
    } else {
      $pls_id = '';
    }
    if(isset($instance['autoplay'])) {
      $autoplay = $instance['autoplay'];
    } else {
      $autoplay = false;
    }
    if(isset($instance['playlist'])) {
      $playlist = $instance['playlist'];
    } else {
      $playlist = false;
    }
    if(isset($instance['cover'])) {
      $cover = $instance['cover'];
    } else {
      $cover = false;
    }
    $pls_list = $wpdb->get_results("SELECT id, name FROM $pls_table_name", ARRAY_A);
    $topweek = $wpdb->get_results("SELECT song_id, SUM(count) AS weekcount FROM $stats_table_name WHERE date > NOW() - INTERVAL 7 DAY GROUP BY song_id ORDER BY weekcount DESC LIMIT 0,10", ARRAY_A);
    $topmonth = $wpdb->get_results("SELECT song_id, SUM(count) AS monthcount FROM $stats_table_name WHERE date > NOW() - INTERVAL 30 DAY GROUP BY song_id ORDER BY monthcount DESC LIMIT 0,10", ARRAY_A);
    $mostPlayed = $wpdb->get_results("SELECT * FROM $songs_table_name ORDER BY `all_count` DESC LIMIT 0,10", ARRAY_A);
    if($mostPlayed){
      if(count($mostPlayed) >= 10){
        array_unshift($pls_list, array("id" => "mostplayed", "name" => "Top 10 of Most Played"));
      }
    }
    if($topmonth){
      if(count($topmonth) >= 10){
        array_unshift($pls_list, array("id" => "topmonth", "name" => "Top 10 of The Month"));
      }
    }
    if($topweek){
      if(count($topweek) >= 10){
        array_unshift($pls_list, array("id" => "topweek", "name" => "Top 10 of The Week"));
      }
    }
    $pls_list = stripslashes_deep($pls_list);
    // Widget admin form
    ?>
    <p>
      <label for="<?php echo $this->get_field_id('title'); ?>"><?php echo 'Title:'; ?></label> 
      <input class="widefat" id="<?php echo $this->get_field_id('title'); ?>" name="<?php echo $this->get_field_name('title'); ?>" type="text" value="<?php echo esc_attr($title); ?>" />
    </p>
    <p>
      <label for="<?php echo $this->get_field_id('pls_id'); ?>"><?php echo 'Select Playlist:'; ?></label> 
      <select class="widefat" name="<?php echo $this->get_field_name('pls_id'); ?>" id="<?php echo $this->get_field_id('pls_id'); ?>">
        <option selected disabled>Select Playlist</option>
      <?php 
        foreach($pls_list as $k => $v){
          echo '<option value="'.$v["id"].'"'.selected($pls_id, $v["id"]).'>'.$v["name"].'</option>';
        }
      ?>
      </select>
    </p>
    <p>
      <input class="checkbox" id="<?php echo $this->get_field_id('autoplay'); ?>" name="<?php echo $this->get_field_name('autoplay'); ?>" type="checkbox" <?php checked($autoplay); ?>/>
      <label for="<?php echo $this->get_field_id('autoplay'); ?>"><?php echo 'Enable Autoplay'; ?></label> 
    </p>
    <p>
      <input class="checkbox" id="<?php echo $this->get_field_id('playlist'); ?>" name="<?php echo $this->get_field_name('playlist'); ?>" type="checkbox"/ <?php checked($playlist);?>>
      <label for="<?php echo $this->get_field_id('playlist'); ?>"><?php echo 'Show Playlist'; ?></label> 
    </p>
    <p>
      <input class="checkbox" id="<?php echo $this->get_field_id('cover'); ?>" name="<?php echo $this->get_field_name('cover'); ?>" type="checkbox"/ <?php checked($cover);?>>
      <label for="<?php echo $this->get_field_id('cover'); ?>"><?php echo 'Show cover'; ?></label> 
    </p>
    <?php 
  }

  // Updating widget replacing old instances with new
  public function update($new_instance, $old_instance) {
    $instance = array();
    $instance['title'] = (!empty($new_instance['title'])) ? strip_tags($new_instance['title']) : '';
    $instance['pls_id'] = (!empty($new_instance['pls_id'])) ? strip_tags($new_instance['pls_id']) : '';
    $instance['autoplay'] = isset($new_instance["autoplay"]) ? true : false;
    $instance['playlist'] = isset($new_instance["playlist"]) ? true : false;
    $instance['cover'] = isset($new_instance["cover"]) ? true : false;
    return $instance;
  }
} // Class wpb_widget ends here
?>